/*************************************************
* Public Key Base Class Header File              *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PK_BASE_H__
#define OPENCL_PK_BASE_H__

#include <opencl/config.h>
#include <opencl/secmem.h>
#include <opencl/exceptn.h>

namespace OpenCL {

/*************************************************
* Public Key Cryptosystem Base Class             *
*************************************************/
class PK_Cryptosystem
   {
   public:
      struct Input_Too_Large : public Exception
         {
         Input_Too_Large(const std::string& where) :
            Exception("PK maximum input length exceeded in " + where) {}
         };

      virtual u32bit max_input() const = 0;
      virtual ~PK_Cryptosystem() {}
   };

/*************************************************
* Public Key Encryptor Base Class                *
*************************************************/
class PK_Encryptor : public PK_Cryptosystem
   {
   public:
      SecureVector<byte> encrypt(const SecureVector<byte>&) const;
      SecureVector<byte> encrypt(const byte[], u32bit) const;
      virtual ~PK_Encryptor() {}
   private:
      virtual SecureVector<byte> enc(const byte[], u32bit) const = 0;
   };

/*************************************************
* Public Key Verifier Base Class                 *
*************************************************/
class PK_Verifier : public PK_Cryptosystem
   {
   public:
      bool valid_signature(const SecureVector<byte>&,
                           const SecureVector<byte>&) const;
      bool valid_signature(const byte[], u32bit,
                           const byte[], u32bit) const;
      virtual ~PK_Verifier() {}
   private:
      virtual bool verify(const byte[], u32bit,
                          const byte[], u32bit) const = 0;
   };

/*************************************************
* Public Key Decryptor Base Class                *
*************************************************/
class PK_Decryptor : public PK_Encryptor
   {
   public:
      SecureVector<byte> decrypt(const SecureVector<byte>&) const;
      SecureVector<byte> decrypt(const byte[], u32bit) const;
      virtual ~PK_Decryptor() {}
   private:
      virtual SecureVector<byte> dec(const byte[], u32bit) const = 0;
   };

/*************************************************
* Public Key Signer Base Class                   *
*************************************************/
class PK_Signer : public PK_Verifier
   {
   public:
      SecureVector<byte> signature(const SecureVector<byte>&) const;
      SecureVector<byte> signature(const byte[], u32bit) const;
      virtual ~PK_Signer() {}
   private:
      virtual SecureVector<byte> sign(const byte[], u32bit) const = 0;
   };

}

#endif
