/*************************************************
* Public Key Ciphers Header File                 *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PUBKEY_H__
#define OPENCL_PUBKEY_H__

#include <opencl/opencl.h>
#include <opencl/pkbase.h>

namespace OpenCL {

/*************************************************
* Message Padding Scheme Base Class              *
*************************************************/
struct MessagePaddingScheme
   {
   virtual SecureVector<byte> pad(const SecureVector<byte>&, u32bit,
                                  RandomNumberGenerator&) const = 0;
   virtual SecureVector<byte> unpad(const SecureVector<byte>&) const = 0;
   virtual u32bit min_padded_size(u32bit) const = 0;
   virtual u32bit max_unpadded_size(u32bit) const = 0;
   virtual ~MessagePaddingScheme() {}
   };

/*************************************************
* Key Derivation Function Base Class             *
*************************************************/
struct KeyDerivationFunction
   {
   virtual SecureVector<byte> derive(const SecureVector<byte>&) const = 0;
   virtual ~KeyDerivationFunction() {}
   };

}

#endif
