/*************************************************
* Public Key Base Class Source File              *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/pk_base.h>

namespace OpenCL {

/*************************************************
* Public Key Encryption                          *
*************************************************/
SecureVector<byte> PK_Encryptor::encrypt(const SecureVector<byte>& in) const
   {
   if(in.size() > max_input()) throw Input_Too_Large("PK_Encryptor::encrypt");
   return enc(in, in.size());
   }

/*************************************************
* Public Key Encryption                          *
*************************************************/
SecureVector<byte> PK_Encryptor::encrypt(const byte in[], u32bit size) const
   {
   if(size > max_input()) throw Input_Too_Large("PK_Encryptor::encrypt");
   return enc(in, size);
   }

/*************************************************
* Public Key Signature Verification              *
*************************************************/
bool PK_Verifier::valid_signature(const SecureVector<byte>& data,
                                  const SecureVector<byte>& sig) const
   {
   if(data.size() > max_input())
      throw Input_Too_Large("PK_Verifier::valid_signature");
   return verify(data, data.size(), sig, sig.size());
   }

/*************************************************
* Public Key Signature Verification              *
*************************************************/
bool PK_Verifier::valid_signature(const byte data[], u32bit data_size,
                                  const byte sig[], u32bit sig_size) const
   {
   if(data_size > max_input())
      throw Input_Too_Large("PK_Verifier::valid_signature");
   return verify(data, data_size, sig, sig_size);
   }

/*************************************************
* Public Key Decryption                          *
*************************************************/
SecureVector<byte> PK_Decryptor::decrypt(const SecureVector<byte>& in) const
   {
   if(in.size() > max_input()) throw Input_Too_Large("PK_Decryptor::decrypt");
   return dec(in, in.size());
   }

/*************************************************
* Public Key Decryption                          *
*************************************************/
SecureVector<byte> PK_Decryptor::decrypt(const byte in[], u32bit size) const
   {
   if(size > max_input()) throw Input_Too_Large("PK_Decryptor::decrypt");
   return dec(in, size);
   }

/*************************************************
* Public Key Signature Generation                *
*************************************************/
SecureVector<byte> PK_Signer::signature(const SecureVector<byte>& in) const
   {
   if(in.size() > max_input()) throw Input_Too_Large("PK_Signer::signature");
   return sign(in, in.size());
   }

/*************************************************
* Public Key Signature Generation                *
*************************************************/
SecureVector<byte> PK_Signer::signature(const byte in[], u32bit size) const
   {
   if(size > max_input()) throw Input_Too_Large("PK_Signer::signature");
   return sign(in, size);
   }


}
